#include <fcntl.h>

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include <fstream>
#include <iostream>

using namespace std;

struct Timer
{
  int start;

  Timer() 
  {
    reset();
  }

  double time( int a, int b )
  {
    return (double)(a - b) / CLOCKS_PER_SEC;
  }

  double reset()
  {
    double old = start;
    return time(start = clock(), old);
  }

  double curT()
  {
    return time(clock(), start);
  }

  void wait( double t )
  {
    reset();
    while (curT() < t)
      ;
  }
};

typedef void Function( const char *file, const char *s, int n );

const int MAX = 4096;
static char buf[MAX];

void F1( const char *file, const char *s, int n )
{
  freopen(file, "wt", stdout);
  for (int i = 0; i < n; i++)
    printf("%s", s);
  fclose(stdout);
}

void F2( const char *file, const char *s, int n )
{
  freopen(file, "wt", stdout);
  for (int i = 0; i < n; i++)
  {
    const char *t = s;
    while (*t)
      putc(*t++, stdout);
  }
  fclose(stdout);
}

void F3( const char *file, const char *s, int n )
{
  int len = strlen(s);
  int cnt = MAX / len;
  for (int i = 0; i < cnt * len; i++)
    buf[i] = s[i % len];

  FILE *F = fopen(file, "wb");
  n /= cnt;
  while (n--)
    fwrite(buf, 1, len * cnt, F);
  fclose(F);
}

void F4( const char *file, const char *s, int n )
{
  ofstream out(file);
  while (n--)
    out << s;
}

void F5( const char *file, const char *s, int n )
{
  freopen(file, "wt", stdout);
  ios_base::sync_with_stdio(0);
  while (n--)
    cout << s;
  cout.flush();
  fclose(stdout);
}

void F6( const char *file, const char *s, int n )
{
  int len = strlen(s);
  int cnt = MAX / len;
  for (int i = 0; i < cnt * len; i++)
    buf[i] = s[i % len];

  int outFile = open(file, O_WRONLY | O_CREAT);
  n /= cnt;
  while (n--)
    write(outFile, buf, len * cnt);
  close(outFile);
}

int main()
{
  system("del /F out*");

  char *s = "Sobaka 10 ";
  Function *F[] = {F1, F2, F3, F4, F5, F6};
  char *fName[] = {"printf", "putc", "fwrite", "ofstream", "cout", "open"};

  int n = sizeof(F) / sizeof(F[0]);
  double *last = new double[n];
  memset(last, 0, sizeof(last[0]) * n);
  double TL = 2.0; // TL in seconds
  int space = 10;
  Timer timer;

  fprintf(stderr, "%*s", space, "value K");
  for (int i = 0; i < n; i++)
    fprintf(stderr, "%*s", space, fName[i]);
  fprintf(stderr, "\n");
   
  for (int k = (int)1e5; k <= (int)1e7; k *= 3)
  {
    fprintf(stderr, "%*d", space, k);
    for (int i = 0; i < n; i++)
    {
      double t = -1;
      if (last[i] <= TL)
      {
        static int no = 0;
        char buf[99];
        sprintf(buf, "out_%02d_%02d", no++, i);

        timer.wait(0.1);
        timer.reset();

        F[i](buf, s, k);
        t = timer.curT();
        last[i] = t;
      }
      fprintf(stderr, "%*.2lf", space, t);
    }
    fprintf(stderr, "\n");
  }
  return 0;
}
                