#include <cmath>

#include <atlimage.h>

CImage im;

void PutLine( int x1, int y1, int x2, int y2 )
{
  HDC hDC = im.GetDC();
  SelectObject(hDC, GetStockObject(BLACK_BRUSH));
  SelectObject(hDC, GetStockObject(BLACK_PEN));
  MoveToEx(hDC, x1, y1, NULL);
  LineTo(hDC, x2, y2);
  im.ReleaseDC();
}

void PutRedHorLine( int x, int y, int w )
{
  // assume bpp = 24 (set in 'Create' function)
  BYTE* ptr = (BYTE*)(im.GetBits()) + y * im.GetPitch() + x * 3;
  for (int i = 0; i < w; i++)
    *ptr++ = 0, *ptr++ = 0, *ptr++ = 0xFF; // assume BGR format in memory

  // image inverted by height. It means that next image line
  // after (0, y) --> (w - 1, y) is (0, y - 1) --> (w - 1, y - 1)
  // and pitch (how much bytes in one line) < 0
}

void fill( BYTE gray )
{
  int w = im.GetWidth(), h = im.GetHeight();

  // see above comments to know why (0, h - 1) is the first in memory
  //memset(im.GetPixelAddress(0, h - 1), gray, abs(im.GetPitch()) * h);

  // this is the same
  memset((BYTE*)im.GetBits() + (h - 1) * im.GetPitch(), gray, abs(im.GetPitch()) * h);
}

int main()
{
  im.Create(800, 600, 24);
  fill(0xFF);

  for (int i = 0; i < 10; i++)
    PutLine(100, 100, 100 + int(50 * sin(i * 0.5)), 100 + int(50 * cos(i * 0.5)));

  PutRedHorLine(200, 200, 100);
  PutRedHorLine(0, 0, 100);

  im.Save("im.bmp"); // big but lossless

  BYTE* pend = (BYTE*)im.GetPixelAddress(im.GetWidth() - 1, 0) + 3;
  BYTE* ptr  = (BYTE*)im.GetPixelAddress(0, im.GetHeight() - 1);
  for (; ptr != pend; ptr++)
    *ptr = 255 - *ptr;
  im.Save("im.jpg"); // small
  return 0;
}
