#define _USE_MATH_DEFINES

#include <cmath>

#include <atlimage.h>

#define forn(i, n) for (int i = 0; i < (int)(n); i++)

CImage im;

void PutLine( int x1, int y1, int x2, int y2 )
{
  HDC hDC = im.GetDC();
//  SelectObject(hDC, CreateSolidBrush(RGB(255,0,67)));
  SelectObject(hDC, CreatePen(PS_SOLID, 1, RGB(255,0,67)));
  MoveToEx(hDC, x1, y1, NULL);
  LineTo(hDC, x2, y2);
  im.ReleaseDC();
}

void PutLine( double x1, double y1, double x2, double y2 )
{
  #define R(x) (int)((x) + 0.5)
  PutLine(R(x1), R(y1), R(x2), R(y2));
}

void fill( BYTE gray )
{
  int w = im.GetWidth(), h = im.GetHeight();
  memset((BYTE*)im.GetBits() + (h - 1) * im.GetPitch(), gray, abs(im.GetPitch()) * h);
}

const int W = 800, H = 600;
const int R = 200;

// a = [-pi/2 .. pi / 2], b = [-pi .. pi]
void GetPoint( double a, double b, double &X, double &Y, double &Z )
{
  X = R * sin(a);
  Y = R * cos(a) * sin(b);
  Z = R * cos(a) * cos(b);
}

const int Na = 10;
const int Nb = 30;

void Rotate( double &x, double &y, double a )
{
  double tx = x * cos(a) - y * sin(a);
  double ty = x * sin(a) + y * cos(a);
  x = tx, y = ty;
}

void GetPoint( int i, int j, double &X, double &Y )
{
  double x, y, z;
  GetPoint(-M_PI / 2 + i * M_PI / Na, -M_PI + j * 2 * M_PI / Nb, x, y, z);
  Rotate(x, y, 1);
  Rotate(x, z, 1.5);
  Rotate(z, y, 2);
  X = W / 2.0 + x, Y = H / 2.0 + z;
}

int main()
{
  im.Create(W, H, 24);
  fill(0xFF);

  forn(i, Na)
    forn(j, Nb)
    {
      double X1, Y1, X2, Y2, X3, Y3;
      GetPoint(i, j, X1, Y1);
      GetPoint(i, j + 1, X2, Y2);
      GetPoint(i + 1, j, X3, Y3);
      PutLine(X1, Y1, X2, Y2);
      PutLine(X1, Y1, X3, Y3);
    }

  im.Save("im.jpg"); // big but lossless
  return 0;
}
