#include <cmath>
#include <cassert>
#include <cstring>
#include <cstdio>

#include <iostream>

using namespace std;

#include <atlimage.h>

#define forn(i, n) for (int i = 0; i < (int)(n); i++)

typedef double dbl;

const dbl eps = 1e-6;

struct pnt
{
  dbl x, y, z;

  pnt( dbl _x, dbl _y, dbl _z ) : x(_x), y(_y), z(_z) { }
  pnt() { }

  pnt operator + ( pnt p ) { return pnt(x + p.x, y + p.y, z + p.z); }
  pnt operator - ( pnt p ) { return pnt(x - p.x, y - p.y, z - p.z); }
  pnt operator * ( dbl k ) { return pnt(x * k, y * k, z * k); }
  pnt operator / ( dbl k ) { return pnt(x / k, y / k, z / k); }
  pnt operator - () { return pnt(-x, -y, -z); }

  pnt operator * ( pnt p )
  {
    return pnt(y * p.z - z * p.y,
               z * p.x - x * p.z,
               x * p.y - y * p.x); 
  }
  dbl operator ^ ( pnt p ) { return x * p.x + y * p.y + z * p.z; }

  bool operator == ( pnt p ) { return fabs(x - p.x) + fabs(y - p.y) + fabs(z - p.z) < eps; }
  bool operator != ( pnt p ) { return fabs(x - p.x) + fabs(y - p.y) + fabs(z - p.z) > eps; }
  bool operator < ( pnt p ) const
  { 
    if (fabs(x - p.x) > eps) return x < p.x;
    if (fabs(y - p.y) > eps) return y < p.y;
    return z < p.z;
  }

  void read() { cin >> x >> y >> z; }

  dbl d2() { return x * x + y * y + z * z; }
  dbl d() { return sqrt(d2()); }

  pnt norm() { return *this / d(); }
};

inline void PutPixel( CImage &im, int x, int y, int r, int g, int b )
{
  BYTE* ptr = (BYTE*)(im.GetBits()) + y * im.GetPitch() + x * 3;
  *ptr++ = b, *ptr++ = g, *ptr++ = r; // assume BGR format in memory
}

void PutLine( CImage &im, int x1, int y1, int x2, int y2 )
{
  HDC hDC = im.GetDC();
  SelectObject(hDC, CreatePen(PS_SOLID, 1, RGB(0, 100, 0)));
  MoveToEx(hDC, x1, y1, NULL);
  LineTo(hDC, x2, y2);
  im.ReleaseDC();
}

int main()
{
  const int SIZE = 500;
  const int M = 100;

  CImage im;
  im.Create(SIZE, SIZE, 24);
  memset((BYTE*)im.GetBits() + (SIZE - 1) * im.GetPitch(), 0xFF, abs(im.GetPitch()) * SIZE);

  pnt A[] = {pnt(200, 200, 500), pnt(-50, -50, 300)};
  dbl R[] = {90, 50};
  int RGB[][3] = {{255, 0, 0}, {0, 0, 255}};

  PutLine(im, 0, SIZE / 2, SIZE, SIZE / 2);
  PutLine(im, SIZE / 2, SIZE, SIZE / 2, 0);

  forn(iy, SIZE)
  {
    forn(ix, SIZE)
    {
      forn(i, 2)
      {
        dbl x = -M + 2 * M * (ix + 0.5) / SIZE;
        dbl y = -M + 2 * M * (iy + 0.5) / SIZE;
        dbl z = 200;
        pnt V = pnt(x, y, z).norm();

        dbl B = -(V ^ A[i]);
        dbl C = (A[i] ^ A[i]) - R[i] * R[i];
        if (B * B - C > 0)
        {
          dbl t = -B - sqrt(B * B - C);
          assert(t > 0);
          pnt p = V * t;
          assert(fabs(((p - A[i]) ^ (p - A[i])) - R[i] * R[i]) < eps);
          dbl co = (p - A[i]).norm() ^ (-V);
          PutPixel(im, ix, iy, RGB[i][0] * co, RGB[i][1] * co, RGB[i][2] * co);
        }
      }
    }
  }

  im.Save("im.bmp");
  return 0;
}
