#include <cstdio>
#include <cassert>

typedef unsigned short WORD;
typedef unsigned int DWORD;
typedef int LONG;

struct BITMAPFILEHEADER
{
  WORD bfType;  //specifies the file type
  DWORD bfSize;  //specifies the size in bytes of the bitmap file
  WORD bfReserved1;  //reserved; must be 0
  WORD bfReserved2;  //reserved; must be 0
  DWORD bOffBits;  //species the offset in bytes from the bitmapfileheader to the bitmap bits
  WORD bfSkip;
};

struct BITMAPINFOHEADER
{
  DWORD biSize;  //specifies the number of bytes required by the struct
  LONG biWidth;  //specifies width in pixels
  LONG biHeight;  //species height in pixels
  WORD biPlanes; //specifies the number of color planes, must be 1
  WORD biBitCount; //specifies the number of bit per pixel
  DWORD biCompression;//spcifies the type of compression
  DWORD biSizeImage;  //size of image in bytes
  LONG biXPelsPerMeter;  //number of pixels per meter in x axis
  LONG biYPelsPerMeter;  //number of pixels per meter in y axis
  DWORD biClrUsed;  //number of colors used by th ebitmap
  DWORD biClrImportant;  //number of colors that are important
};

unsigned char *LoadBitmapFile(const char *filename, BITMAPINFOHEADER *bitmapInfoHeader)
{
  BITMAPFILEHEADER bitmapFileHeader; //our bitmap file header

  //open filename in read binary mode
  FILE *filePtr = fopen(filename,"rb");
  if (filePtr == NULL)
    return NULL;

  //read the bitmap file header
  fread(&bitmapFileHeader, sizeof(BITMAPFILEHEADER),1,filePtr);

  //verify that this is a bmp file by check bitmap id
  if (bitmapFileHeader.bfType != 0x4D42)
  {
    fclose(filePtr);
    return NULL;
  }

  printf("%d\n", sizeof(bitmapFileHeader));
  printf("type = 0x%X\n", bitmapFileHeader.bfType);
  printf("size = %d\n", bitmapFileHeader.bfSize);

  for (int i = 0; i < (int)sizeof(bitmapFileHeader); i++)
    printf("0x%X\n", *((unsigned char *)&bitmapFileHeader + i));

  printf("%d\n", sizeof(bitmapFileHeader));
/*
  #define OUT(bfType) \
    printf("offset = %d, size = %d\n", (char *)&bitmapFileHeader.bfType - (char *)&bitmapFileHeader, sizeof(bitmapFileHeader.bfType));
  OUT(bfType)
  OUT(bfSize)
  OUT(bfReserved1)
  OUT(bfReserved2)
  OUT(bOffBits)
*/

  //read the bitmap info header
  fread(bitmapInfoHeader, sizeof(BITMAPINFOHEADER),1,filePtr);

  //move file point to the begging of bitmap data
  fseek(filePtr, bitmapFileHeader.bOffBits, SEEK_SET);

  //allocate enough memory for the bitmap image data
  unsigned char *bitmapImage = new unsigned char[bitmapInfoHeader->biSizeImage];

  //verify memory allocation
  if (!bitmapImage)
  {
    delete [] bitmapImage;
    fclose(filePtr);
    return NULL;
  }

  //read in the bitmap image data
  fread(bitmapImage,bitmapInfoHeader->biSizeImage,1,filePtr);

  //make sure bitmap image data was read
  if (bitmapImage == NULL)
  {
    fclose(filePtr);
    return NULL;
  }

  //swap the r and b values to get RGB (bitmap is BGR)
  for (int imageIdx = 0; imageIdx < (int)bitmapInfoHeader->biSizeImage; imageIdx+=3)
  {
    unsigned char tempRGB = bitmapImage[imageIdx];
    bitmapImage[imageIdx] = bitmapImage[imageIdx + 2];
    bitmapImage[imageIdx + 2] = tempRGB;
  }

  //close file and return bitmap iamge data
  fclose(filePtr);
  return bitmapImage;
}

int main()
{
  BITMAPINFOHEADER bitmapInfoHeader;
  unsigned char *bitmapData = LoadBitmapFile("a.bmp",&bitmapInfoHeader);
  assert(bitmapData);
  printf("size = %d\n", (int)bitmapInfoHeader.biSize);
  printf("w = %d\n", (int)bitmapInfoHeader.biWidth);
  printf("h = %d\n", (int)bitmapInfoHeader.biHeight);
  return 0;
}

/*
  DWORD biSize;  //specifies the number of bytes required by the struct
  LONG biWidth;  //specifies width in pixels
  LONG biHeight;  //species height in pixels
  WORD biPlanes; //specifies the number of color planes, must be 1
  WORD biBitCount; //specifies the number of bit per pixel
  DWORD biCompression;//spcifies the type of compression
  DWORD biSizeImage;  //size of image in bytes
  LONG biXPelsPerMeter;  //number of pixels per meter in x axis
  LONG biYPelsPerMeter;  //number of pixels per meter in y axis
  DWORD biClrUsed;  //number of colors used by th ebitmap
  DWORD biClrImportant;  //number of colors that are important
*/