#include <cstdio>
#include <cassert>

typedef unsigned short WORD;
typedef unsigned int DWORD;
typedef int LONG;
typedef unsigned char BYTE;

struct BITMAPFILEHEADER
{
  WORD bfType : 16;  //specifies the file type
  DWORD bfSize : 32;  //specifies the size in bytes of the bitmap file
  WORD bfReserved1 : 16;  //reserved; must be 0
  WORD bfReserved2 : 16;  //reserved; must be 0
  DWORD bOffBits : 32;  //species the offset in bytes from the bitmapfileheader to the bitmap bits
  WORD bfReserved3;  //reserved; must be 0
};

struct BITMAPINFOHEADER
{
  DWORD biSize;  //specifies the number of bytes required by the struct
  LONG biWidth;  //specifies width in pixels
  LONG biHeight;  //species height in pixels
  WORD biPlanes; //specifies the number of color planes, must be 1
  WORD biBitCount; //specifies the number of bit per pixel
  DWORD biCompression;//spcifies the type of compression
  DWORD biSizeImage;  //size of image in bytes
  LONG biXPelsPerMeter;  //number of pixels per meter in x axis
  LONG biYPelsPerMeter;  //number of pixels per meter in y axis
  DWORD biClrUsed;  //number of colors used by th ebitmap
  DWORD biClrImportant;  //number of colors that are important
};

unsigned char *LoadBitmapFile(const char *filename, BITMAPINFOHEADER *bitmapInfoHeader)
{
  BYTE bitmapFileHeader[14]; //our bitmap file header

  //open filename in read binary mode
  FILE *filePtr = fopen(filename,"rb");
  if (filePtr == NULL)
    return NULL;

  //read the bitmap file header
  fread(bitmapFileHeader, 14, 1, filePtr);
  #define GET(type, offset) *(type *)(bitmapFileHeader + offset)

  //verify that this is a bmp file by check bitmap id
  if (GET(WORD, 0) != 0x4D42)
  {
    fclose(filePtr);
    return NULL;
  }

  int size = GET(DWORD, 2), bn = GET(DWORD, 10);
  printf("type = 0x%X\n", GET(WORD, 0));
  printf("size = %d\n", size);

  //read the bitmap info header
  fread(bitmapInfoHeader, sizeof(BITMAPINFOHEADER),1,filePtr);
  for (int i = 0; i < sizeof(BITMAPINFOHEADER); i++)
    printf("0x%X\n", *((BYTE *)bitmapInfoHeader + i));
  #define OUT(bfType) \
    printf("offset = %d, size = %d\n", (char *)&bitmapInfoHeader->bfType - (char *)bitmapInfoHeader, sizeof(bitmapInfoHeader->bfType));
  OUT(biSize)
  OUT(biWidth)
  OUT(biHeight)

  //move file point to the begging of bitmap data
  fseek(filePtr, bn, SEEK_SET);

  //allocate enough memory for the bitmap image data
  unsigned char *bitmapImage = new unsigned char[bitmapInfoHeader->biSizeImage];

  //verify memory allocation
  if (!bitmapImage)
  {
    delete [] bitmapImage;
    fclose(filePtr);
    return NULL;
  }

  //read in the bitmap image data
  fread(bitmapImage,bitmapInfoHeader->biSizeImage,1,filePtr);

  printf("%d\n", sizeof(*bitmapInfoHeader));
  printf("%d\n", bitmapInfoHeader->biSizeImage);

  //make sure bitmap image data was read
  if (bitmapImage == NULL)
  {
    fclose(filePtr);
    return NULL;
  }

  printf("%d\n", bitmapInfoHeader->biSizeImage);

  //swap the r and b values to get RGB (bitmap is BGR)
  for (int imageIdx = 0; imageIdx < (int)bitmapInfoHeader->biSizeImage; imageIdx+=3)
  {
    unsigned char tempRGB = bitmapImage[imageIdx];
    bitmapImage[imageIdx] = bitmapImage[imageIdx + 2];
    bitmapImage[imageIdx + 2] = tempRGB;
  }

  printf("%d\n", bitmapInfoHeader->biSizeImage);

  //close file and return bitmap iamge data
  fclose(filePtr);

  for (int imageIdx = 0; imageIdx < (int)bitmapInfoHeader->biSizeImage; imageIdx+=3)
  {
    bitmapImage[imageIdx] = 255;
  }

  FILE *f = fopen("b.bmp","wb");
  fwrite(bitmapFileHeader, 14, 1, f);
  fwrite(bitmapInfoHeader, sizeof(BITMAPINFOHEADER),1,f);

  printf("%d\n", bitmapInfoHeader->biSizeImage);

  fwrite(bitmapImage,bitmapInfoHeader->biSizeImage,1,f);


  return bitmapImage;
}

int main()
{
  BITMAPINFOHEADER bitmapInfoHeader;
  unsigned char *bitmapData = LoadBitmapFile("a.bmp",&bitmapInfoHeader);
  assert(bitmapData);
  printf("size = %d\n", (int)bitmapInfoHeader.biSize);
  printf("w = %d\n", (int)bitmapInfoHeader.biWidth);
  printf("h = %d\n", (int)bitmapInfoHeader.biHeight);
  printf("size = 0x%X\n", (int)bitmapInfoHeader.biSize);
  printf("w = 0x%X\n", (int)bitmapInfoHeader.biWidth);
  printf("h = 0x%X\n", (int)bitmapInfoHeader.biHeight);
  return 0;
}

/*
  DWORD biSize;  //specifies the number of bytes required by the struct
  LONG biWidth;  //specifies width in pixels
  LONG biHeight;  //species height in pixels
  WORD biPlanes; //specifies the number of color planes, must be 1
  WORD biBitCount; //specifies the number of bit per pixel
  DWORD biCompression;//spcifies the type of compression
  DWORD biSizeImage;  //size of image in bytes
  LONG biXPelsPerMeter;  //number of pixels per meter in x axis
  LONG biYPelsPerMeter;  //number of pixels per meter in y axis
  DWORD biClrUsed;  //number of colors used by th ebitmap
  DWORD biClrImportant;  //number of colors that are important
*/