#include <cstdio>
#include <cmath>

typedef double (*func)( double );

class Func
{
  func f;
public:
  Func() { }
  Func( func _f ) : f(_f) { }

  virtual double calc( double x ) const
  {
    return f(x);
  }
};

class SumFunc : public Func
{
  const Func *a, *b;
public:
  SumFunc( Func *_a, Func *_b ) : a(_a), b(_b) { }

  double calc( double x )
  {
    return a->calc(x) + b->calc(x);
  }
};

class DifFunc : public Func
{
public:
  Func *f;

  DifFunc( Func *_f ) : f(_f) { }

  double calc( double x )
  {
    double e = 1e-6;
    return (f->calc(x + e) - f->calc(x - e)) / (2 * e);
  }
};

Func* operator + ( Func &a, Func &b )
{
  return new SumFunc(&a, &b);
}

int main()
{
  DifFunc d(*new Func(cos) + *new Func(sin));

  printf("result : d(10)=%lf\n", d.calc(10)); // d(x) = (cos + sin)'(x)
  printf("answer : d(10)=%lf\n", -sin(10.0) + cos(10.0)); 
  return 0;
}
