#include <cstdio>
#include <cmath>

class Func
{
public:
  typedef double (*func)( double );

  func f;

  Func() { }
  Func( func _f ) { f = _f; }

  virtual double calc( double x ) const
  {
    return f(x);
  }
};

#define FUNC(name, op, calc_body) \
class name : public Func \
{ \
public: \
  Func *a, *b; \
  name( Func *_a, Func *_b ) { a = _a, b = _b; } \
  double calc( double x ) const { calc_body } \
}; \
Func* operator op ( Func &a, Func &b ) \
{ \
  return new name(&a, &b); \
}                                                                                        

FUNC(AddFunc, +, return a->calc(x) + b->calc(x);)
FUNC(SubFunc, -, return a->calc(x) - b->calc(x);)
FUNC(MulFunc, *, return a->calc(x) * b->calc(x);)
FUNC(DivFunc, /, return a->calc(x) / b->calc(x);)

class DifFunc : public Func
{
public:
  Func *f;

  DifFunc( Func *_f ) { f = _f; }

  double calc( double x )
  {
    double e = 1e-6;
    return (f->calc(x + e) - f->calc(x - e)) / (2 * e);
  }
};

int main()
{
  DifFunc d(*new Func(cos) + *new Func(sin));

  printf("result : d(10)=%lf\n", d.calc(10)); // d(x) = (cos + sin)'(x)
  printf("answer : d(10)=%lf\n", -sin(10.0) + cos(10.0)); 
  return 0;
}
