#define _CRT_SECURE_NO_DEPRECATE
#undef UNICODE

#include <windows.h>

#include <cstdarg>
#include <cstdio>
#include <cstring>
#include <cassert>

#include <string>

using namespace std;

#define forn(i, n) for (int i = 0; i < (int)(n); i++)

STARTUPINFO sa[2];
PROCESS_INFORMATION pi[2];

HANDLE hStdinRd[2], hStdinWr[2];
HANDLE hStdoutRd[2], hStdoutWr[2];
SECURITY_ATTRIBUTES hsaStdin[2], hsaStdout[2];

void ErrorExit( string s )
{
    TCHAR szBuf[80]; 
    LPVOID lpMsgBuf;
    DWORD dw = GetLastError(); 

    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf,
        0, NULL);

    wsprintf(szBuf, "%s | code = %d | error = %s", s.c_str(), dw, lpMsgBuf);
 
    MessageBox(NULL, szBuf, "Error", MB_OK); 

    LocalFree(lpMsgBuf);
    ExitProcess(dw);
}

char ReadChar( HANDLE h )
{
	char c;
	DWORD len;
	if (!ReadFile(h, &c, 1, &len, 0))
		ErrorExit("Read error");
	assert(len == 1);
	return c;
}

int ReadInt( HANDLE h )
{
	char c = ReadChar(h);
	while (!isdigit(c))
		c = ReadChar(h);

	int x = 0;
	while (isdigit(c))
		x = x * 10 + c - '0', c = ReadChar(h);
	return x;
}

void PrintH( HANDLE h, ... )
{
	va_list ap;
	va_start(ap, h);

	char *format = va_arg(ap, char *);
	static char buf[999];
	vsprintf(buf, format, ap);

	DWORD len, bufl = (DWORD)strlen(buf);
	if (!WriteFile(h, buf, bufl, &len, 0))
		ErrorExit("WriteFile");
	assert(len == bufl);
}

void LoadGame( char *name[2] )
{
	for (int i = 0; i < 2; i++)
	{
		fprintf(stderr, "Create process %d : %s\n", i, name[i]);

		hsaStdin[i].nLength = sizeof(SECURITY_ATTRIBUTES);
		hsaStdin[i].bInheritHandle = TRUE; 
		hsaStdin[i].lpSecurityDescriptor = NULL; 
		if (!CreatePipe(&hStdinRd[i], &hStdinWr[i], &hsaStdin[i], 0))
			ErrorExit("CreatePipe for stdin");

		hsaStdout[i].nLength = sizeof(SECURITY_ATTRIBUTES);
		hsaStdout[i].bInheritHandle = TRUE; 
		hsaStdout[i].lpSecurityDescriptor = NULL; 
		if (!CreatePipe(&hStdoutRd[i], &hStdoutWr[i], &hsaStdout[i], 0))
			ErrorExit("CreatePipe for stdout");

		memset(&sa[i], 0, sizeof(sa[i]));
		sa[i].cb = sizeof(STARTUPINFO);
		sa[i].hStdInput = hStdinRd[i];
		sa[i].hStdOutput = hStdoutWr[i];
		sa[i].hStdError = GetStdHandle(STD_ERROR_HANDLE);
		sa[i].dwFlags |= STARTF_USESTDHANDLES;

		if (!CreateProcess(name[i], 0, 0, 0, TRUE, 0, 0, 0, &sa[i], &pi[i]))
			ErrorExit("CreateProcess");
	}
}

const int W = 15;
const int H = 15;
char ch[] = {'x', 'o'};
char board[H][W + 1];

void StartGame()
{
	printf("%d %d\n", W, H);
	forn(i, H)
		forn(j, W + 1)
			board[i][j] = (j == W ? 0 : '.');
	forn(i, 2)
		PrintH(hStdinWr[i], "%d %d %c\n", W, H, ch[i]);
}

int Win( char ch )
{
	int dx[] = {1, 1, 1, 0};
	int dy[] = {-1, 0, 1, 1};
	forn(i, H)
		forn(j, W)
			forn(d, 4)
			{	
				int x = j, y = i, bad = 0;
				forn(k, 5)
				{
					if (!(0 <= x && x < W && 0 <= y && y < H && board[y][x] == ch))
						bad = 1;
					x += dx[d], y += dy[d];
				}
				if (!bad)
					return 1;
			}
	return 0;
}

int x, y, cc = 0, cnt = 0;

DWORD WINAPI ReadMove( void * )
{
	x = ReadInt(hStdoutRd[cc]);
	y = ReadInt(hStdoutRd[cc]);
	return 0;
}

int main( int argc, char *argv[] )
{
	if (argc < 3)
	{
		puts("Usage: <player1> <player2>");
		puts("Players - executable files.");
		return 0;
	}

	char *name[] = {argv[1], argv[2]};
	LoadGame(name);
	
	fprintf(stderr, "The game starts!\n");
	StartGame();
	
	enum {OK, PE, TL, DRAW};
	int res = DRAW, first[2] = {1, 1};

	forn(i, 2)
		 puts(name[i]);

	int num = 0;
	while (++num <= W * H)
	{
		fprintf(stderr, "Player %d [%s] moves [%d].\n", cc + 1, name[cc], ++cnt);

		forn(i, H)
			PrintH(hStdinWr[cc], "%s\n", board[i]);

		int TimeLimit = first[cc] ? 5000 : 250;
		first[cc] = 0;
		HANDLE th = CreateThread(0, 0, ReadMove, 0, 0, 0);
		if (WaitForSingleObject(th, TimeLimit) == WAIT_TIMEOUT)
		{
			TerminateThread(th, 0);
			fprintf(stderr, "%d [%s] : Time Limit [%d] was Exceeded.\n", cc + 1, name[cc], TimeLimit);
			res = TL;
			break;
		}

		if (!(0 <= x && x < W && 0 <= y && y < H))
		{
			fprintf(stderr, "%d [%s] : %d, %d - invalid move\n", cc + 1, name[cc], x, y);
			res = PE;
			break;
		}
		if (board[y][x] != '.')
		{
			fprintf(stderr, "%d [%s] : %d, %d - cell is already occuppied\n", cc + 1, name[cc], x, y);
			res = PE;
			break;
		}
		printf("%d %d\n", x, y);

		fprintf(stderr, "move = %d, %d\n", x, y);
		board[y][x] = ch[cc];
		if (Win(ch[cc]))
		{
			fprintf(stderr, "%d [%s] : Win!\n", cc + 1, name[cc]);
			res = OK;
			break;
		}

		cc ^= 1;
	}
	printf("-1 -1 %d\n", res);
	if (res == DRAW)
	  fprintf(stderr, "Draw :-D\n");
	if (res == DRAW)
	  puts("Draw");
	else if ((res == OK) ^ (cc & 1))
	  puts("Win");
	else
	  puts("Lose");
	
	fprintf(stderr, "Terminate processes...\n");
	for (int i = 0; i < 2; i++)
	{
		TerminateProcess(pi[i].hProcess, 0);
		WaitForSingleObject(pi[i].hProcess, 3000);
	}
	return 0;
}
