#include <math.h>

#define M_PI 3.14159265358979

#include "tgl.h"

#include <stdlib.h>
#include <time.h>

#define MY_RGB(R, G, B) RGB(B, G, R)

/* Tormoz is tormoz */
#define TORMOZ 30

/* Size of window */
#define WINDOWX 700
#define WINDOWY 700

#define SDVIK 5
#define RADIUS 200
#define ARROW_RADIUS 20
#define MOUSE_RADIUS 300

/* Speed of mouse */
#define STEP1 0.05

#define BACK_COLOR MY_RGB(255, 255, 0)
#define COLOR1 MY_RGB(255, 0, 0)
#define COLOR2 MY_RGB(0, 0, 255)
#define STR_COLOR MY_RGB(0, 255, 255)

/* The buffer wich now is visible */
INT VisibleBuf = 0;

/* The buffer of coordinates of pixels for the drawing arrows by 2 points */
INT BufCoor[1][2] = { { WINDOWX / 2, WINDOWY / 2 } };

/* Number of arrows */
INT N = sizeof(BufCoor) / (sizeof(INT) * 2);

/* Puts the line to the buffer */
VOID PutLine( INT X, INT Y, INT X1, INT Y1, COLORREF Color1, COLORREF Color2 )
{
	INT A = 0, i, L;
  BYTE R1 = (BYTE)(Color1), G1 = (BYTE)(Color1 >> 8), B1 = (BYTE)(Color1 >> 16);
	BYTE R2 = (BYTE)(Color2), G2 = (BYTE)(Color2 >> 8), B2 = (BYTE)(Color2 >> 16);
	DOUBLE k, b, koR, koG, koB, R3 = R1, G3 = G1, B3 = B1;
	
  if (X == X1)
  {
    if (Y > Y1)
      COM_SWAP(Y, Y1, A);
    if (Color1 == Color2)
		  for (i = Y; i <= Y1; i++)
        TGL_PutPixel(X, i, Color1);
		else
		{
			L = Y1 - Y + 1;	
			koR = (DOUBLE)(R2 - R3) / L;
			koG = (DOUBLE)(G2 - G3) / L;
			koB = (DOUBLE)(B2 - B3) / L;
			for (i = Y; i <= Y1; i++)
			{
				TGL_PutPixel(X, i, RGB(R3, G3, B3));
				R3 += koR;
				G3 += koG;
				B3 += koB;
			}
		}
		return; 
  }
  if (Y == Y1)
  {
    if (X > X1)
      COM_SWAP(X, X1, A);
    if (Color1 == Color2)
			for (i = X; i <= X1; i++)
				TGL_PutPixel(i, Y, Color1);
		else
		{
			L = X1 - X + 1;	
			koR = (DOUBLE)(R2 - R3) / L;
			koG = (DOUBLE)(G2 - G3) / L;
			koB = (DOUBLE)(B2 - B3) / L;
			for (i = X; i <= X1; i++)
			{
				TGL_PutPixel(i, Y, RGB(R3, G3, B3));
				R3 += koR;
				G3 += koG;
				B3 += koB;
			}
		}
    return; 
  }
  k = ((DOUBLE)(Y1 - Y) / (X1 - X));
  b = Y - X * k;
 	if (COM_ABS(k) < 1)
	{
	  if (X > X1)
  	{
		  COM_SWAP(X, X1, A);
			COM_SWAP(Y, Y1, A);
		}
		if (Color1 == Color2)
			for (i = X; i <= X1; i++)
				TGL_PutPixel(i, (INT)(i * k + b), Color1);
		else
		{
			L = X1 - X + 1;	
			koR = (DOUBLE)(R2 - R3) / L;
			koG = (DOUBLE)(G2 - G3) / L;
			koB = (DOUBLE)(B2 - B3) / L;
			for (i = X; i <= X1; i++)
			{
				TGL_PutPixel(i, (INT)(i * k + b), RGB(R3, G3, B3));
				R3 += koR;
				G3 += koG;
				B3 += koB;
			}
		}
	}
	else
	{
		if (Y > Y1)
		{
	 	  COM_SWAP(X, X1, A);
		  COM_SWAP(Y, Y1, A);
		}
		if (Color1 == Color2)
			for (i = Y; i <= Y1; i++)
				TGL_PutPixel((INT)((i - b) / k), i, Color1);
		else
		{
			L = Y1 - Y + 1;	
			R3 = R1;	
			G3 = G1;	
			B3 = B1;	
			koR = (DOUBLE)(R2 - R3) / L;
			koG = (DOUBLE)(G2 - G3) / L;
			koB = (DOUBLE)(B2 - B3) / L;
			for (i = Y; i <= Y1; i++)
			{
				TGL_PutPixel((INT)((i - b) / k), i, RGB(R3, G3, B3));
				R3 += koR;
				G3 += koG;
				B3 += koB;
			}
		}
	}
}

/* Puts the arrow */
VOID DrawArrow( INT X, INT Y, INT L1, INT L2, DOUBLE Angle, DOUBLE Angle2, COLORREF Color, COLORREF Color2 )
{
	INT X1 = (INT)(X + L1 * cos(Angle)), Y1 = (INT)(Y + L1 * sin(-Angle));

	PutLine(X, Y, X1, Y1, Color, Color2);
	Angle = 2 * M_PI - Angle;
	PutLine(X1, Y1, (INT)(X1 - L2 * cos(Angle + Angle2)), (INT)(Y1 - L2 * sin(Angle + Angle2)), Color2, Color);
	PutLine(X1, Y1, (INT)(X1 - L2 * cos(Angle - Angle2)), (INT)(Y1 - L2 * sin(Angle - Angle2)), Color, Color2);
}

/* Puts the arrow by 2 points */
VOID DrawArrowBy2Points( INT X, INT Y, INT X1, INT Y1, INT L, DOUBLE Angle2, COLORREF Color )
{
	INT W = X1 - X, H = Y1 - Y, L1 = (INT)sqrt(W * W + H * H);
	DOUBLE Angle1 = acos((DOUBLE)W / L1);
	
	if (H > 0)
		Angle1 = 2 * M_PI - Angle1; 
	if (L > L1)
		L = L1;
	DrawArrow(X, Y, L1, L, Angle1, Angle2, Color, Color);
}

/* Draws the compas */
VOID DrawCompas( INT X, INT Y, INT R, INT MX, INT MY, COLORREF BackColor, COLORREF Color1, COLORREF Color2, DOUBLE Bright )
{
	INT W = MX - X, H = MY - Y, L = (INT)sqrt(W * W + H * H);
	BYTE R1 = (BYTE)(BackColor), G1 = (BYTE)(BackColor >> 8), B1 = (BYTE)(BackColor >> 16);
	BYTE R2 = (BYTE)(Color1), G2 = (BYTE)(Color1 >> 8), B2 = (BYTE)(Color1 >> 16);
	BYTE R3 = (BYTE)(Color2), G3 = (BYTE)(Color2 >> 8), B3 = (BYTE)(Color2 >> 16);
	DOUBLE K, Angle;

	if ((K = (DOUBLE)COM_ABS(L) / Bright) < 1)
	{
		R2 = (BYTE)(K * R2 + (1 - K) * R1);
		G2 = (BYTE)(K * G2 + (1 - K) * G1);
		B2 = (BYTE)(K * B2 + (1 - K) * B1);
		R3 = (BYTE)(K * R3 + (1 - K) * R1);
		G3 = (BYTE)(K * G3 + (1 - K) * G1);
		B3 = (BYTE)(K * B3 + (1 - K) * B1);
	}
	Color1 = RGB(R2, G2, B2);
  Color2 = RGB(R3, G3, B3);
	Angle = acos((DOUBLE)W / L);
	if (H > 0)
		Angle = 2 * M_PI - Angle; 
	if ((DOUBLE)L / R < 1)
		R = L;
	DrawArrow(X, Y, R, R / 3, Angle, 0.6, Color2, Color1);
	DrawArrow(X, Y, -R, -R / 3, Angle, 0.6, Color1, Color2);
}

INT WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow )
{
	POINT P;
	INT i, j, Mx, My, NewX, NewY, CursorX, CursorY, StartX = 350, StartY = 350;
	DOUBLE teta = 0;

	CursorX = GetSystemMetrics(SM_CXCURSOR);
	CursorY = GetSystemMetrics(SM_CYCURSOR);
	NewX = WINDOWX / 2;
	NewY = WINDOWY / 2;
	SetCursorPos(NewX, NewY);
	TGL_Init("TitleSK", WINDOWX, WINDOWY, 150);
	while (!GetAsyncKeyState(VK_ESCAPE))
	{ 
		TGL_Clear();
		GetCursorPos(&P);
		StartX += P.x - NewX;
		StartY += P.y - NewY;
		teta += STEP1;
		NewX = (int)(StartX + MOUSE_RADIUS * cos(teta) * sin(teta));
		NewY = (int)(StartY + MOUSE_RADIUS * sin(teta));
		SetCursorPos(NewX, NewY);
		TGL_GetMouseCoor(&P);
		Mx = P.x + CursorX / 2;
		My = P.y + CursorY / 2;
		for (j = ARROW_RADIUS; j <= WINDOWY - ARROW_RADIUS; j += ARROW_RADIUS * 2 + SDVIK)
		  for (i = ARROW_RADIUS; i <= WINDOWX - ARROW_RADIUS; i += ARROW_RADIUS * 2 + SDVIK)
		    DrawCompas(i, j, ARROW_RADIUS, Mx, My, BACK_COLOR, COLOR1, COLOR2, RADIUS);
		TGL_Update();		
		Sleep(TORMOZ);
	}
}
