#include <stdio.h>

#include "tgl.h"

#define MY_CLASS_NAME "TGL Window"

HANDLE TGL_hEVENT;
HWND TGL_hWnd;

BYTE Back;
CHAR Title[100];
VOID (*YourFunc)( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam );

struct 
{
  HBITMAP hBuf;
  BYTE *BufBits;
  HDC hMemDC;
  BITMAPINFOHEADER Head;
} Ex;

int W, H;

static BOOL Win_OnCreate( HWND hWnd, CREATESTRUCT *CS )
{
  HDC hDC;
  RECT rc;

  W = ((INT *)CS->lpCreateParams)[0];
  H = ((INT *)CS->lpCreateParams)[1];
  memset(&Ex, 0, sizeof(Ex));

  hDC = GetDC(NULL);
  Ex.hMemDC = CreateCompatibleDC(hDC);
  ReleaseDC(NULL, hDC);

  Ex.Head.biSize = sizeof(BITMAPINFOHEADER);
  Ex.Head.biBitCount = 32;
  Ex.Head.biWidth = W;
  Ex.Head.biHeight = -H;
  Ex.Head.biPlanes = 1;
  Ex.hBuf = CreateDIBSection(NULL, (BITMAPINFO *)&Ex.Head, DIB_RGB_COLORS, (VOID *)&Ex.BufBits, NULL, 0);

  SelectObject(Ex.hMemDC, Ex.hBuf);

  rc.left = CS->x;
  rc.right = CS->x + W;
  rc.top = CS->y;
  rc.bottom = CS->y + H;
  AdjustWindowRect(&rc, CS->style, FALSE);
  MoveWindow(hWnd, rc.left, rc.top, W, H, TRUE);
  return TRUE;
}

VOID TGL_WinOnDestroy( HWND hWnd )
{
  DeleteDC(Ex.hMemDC);
  DeleteDC(Ex.hMemDC);
  PostQuitMessage(0);
  exit(0);
}

BOOL Win_OnEraseBkgnd( HWND hWnd, HDC hDC )
{
  BitBlt(hDC, 0, 0, W, H, Ex.hMemDC, 0, 0, SRCCOPY);
  return TRUE;
}

LRESULT CALLBACK TGL_GraphWinFunc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
  switch (Msg)
  {
  HANDLE_MSG(hWnd, WM_CREATE, Win_OnCreate);
  HANDLE_MSG(hWnd, WM_DESTROY, TGL_WinOnDestroy);
  HANDLE_MSG(hWnd, WM_ERASEBKGND, Win_OnEraseBkgnd);
  }
  if (YourFunc)
    YourFunc(hWnd, Msg, wParam, lParam);
  return DefWindowProc(hWnd, Msg, wParam, lParam);
}

DWORD WINAPI TGL_MainThread( VOID *Params )
{
  MSG msg;
  
  TGL_hWnd = CreateWindow(MY_CLASS_NAME, Title, WS_OVERLAPPED, 10, 30, 
    CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, GetModuleHandle(NULL), Params);
  ShowWindow(TGL_hWnd, SW_SHOWNORMAL);
  UpdateWindow(TGL_hWnd);
  SetEvent(TGL_hEVENT);
  while (TRUE)
    if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
    {
      if (msg.message == WM_QUIT)
        break;
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
  TGL_hWnd = NULL;
  return msg.wParam;
}
 
BOOL TGL_Init( CHAR *T, INT Width, INT Heigth, BYTE BackColor )
{
  DWORD ThID;
  HANDLE TGL_hMainThread;
  WNDCLASSEX wc;
  INT Params[2] = {Width, Heigth};

  Back = BackColor;
  strcpy(Title, T);
  if (TGL_hWnd != NULL)
    return FALSE;
  TGL_hEVENT = CreateEvent(NULL, FALSE, FALSE, NULL);
  wc.cbSize = sizeof(WNDCLASSEX);
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = sizeof(VOID *);
  wc.hbrBackground = GetStockObject(BLACK_BRUSH);
  wc.hCursor = NULL;
  wc.hIcon = NULL;
  wc.hIconSm = NULL;
  wc.hInstance = GetModuleHandle(NULL);
  wc.lpszClassName = MY_CLASS_NAME;
  wc.lpszMenuName = NULL;
  wc.lpfnWndProc = TGL_GraphWinFunc;
  RegisterClassEx(&wc);

  TGL_hMainThread = CreateThread(NULL, 0, TGL_MainThread, Params, 0, &ThID);
  WaitForSingleObject(TGL_hEVENT, INFINITE);
  CloseHandle(TGL_hEVENT);
  return TRUE;
}

VOID TGL_Update( VOID )
{
  HDC hDC;

  hDC = GetDC(TGL_hWnd);
  BitBlt(hDC, 0, 0, W, H, Ex.hMemDC, 0, 0, SRCCOPY);
  ReleaseDC(TGL_hWnd, hDC);
}

VOID TGL_PutPixel( INT X, INT Y, COLORREF Color )
{
  if (X < 0 || Y < 0 || X >= W || Y >= H)
    return;
  *((DWORD *)Ex.BufBits + W * Y + X) = Color;
}

VOID TGL_Clear( VOID )
{
  memset(Ex.BufBits, Back, W * H * 4);
}

VOID TGL_PutWindowText( CHAR *Str )
{
  static char Buf[10013];

  sprintf(Buf, "%s %s", Title, Str);
  SetWindowText(TGL_hWnd, Buf);
}

VOID TGL_GetMouseCoor( POINT *P )
{
  GetCursorPos(P);
  ScreenToClient(TGL_hWnd, P);
}

VOID TGL_SetWindowHandle( VOID *Func )
{
  YourFunc = Func;
}
